@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#include "galias.h"
 #include "glib.h"
 
 #include <sys/stat.h>
@@ -348,7 +349,13 @@ g_file_error_from_errno (gint err_no)
       return G_FILE_ERROR_PERM;
       break;
 #endif
-      
+
+#ifdef ENOSYS
+    case ENOSYS:
+      return G_FILE_ERROR_NOSYS;
+      break;
+#endif
+
     default:
       return G_FILE_ERROR_FAILED;
       break;
