@@ -36,6 +36,7 @@
 /* uncomment the next line to get poll() debugging info */
 /* #define G_MAIN_POLL_DEBUG */
 
+#include "galias.h"
 #include "glib.h"
 #include "gthreadinit.h"
 #include <signal.h>
@@ -265,7 +266,7 @@ enum {
   CHILD_WATCH_INITIALIZED_THREADED
 };
 static gint child_watch_init_state = CHILD_WATCH_UNINITIALIZED;
-static gint child_watch_count = 1;
+static gint child_watch_count = 0;
 static gint child_watch_wake_up_pipe[2] = {0, 0};
 #endif /* !G_OS_WIN32 */
 G_LOCK_DEFINE_STATIC (main_context_list);
@@ -3565,17 +3566,12 @@ g_child_watch_signal_handler (int signum)
 static void
 g_child_watch_source_init_single (void)
 {
-  struct sigaction action;
-
   g_assert (! g_thread_supported());
   g_assert (child_watch_init_state == CHILD_WATCH_UNINITIALIZED);
 
   child_watch_init_state = CHILD_WATCH_INITIALIZED_SINGLE;
 
-  action.sa_handler = g_child_watch_signal_handler;
-  sigemptyset (&action.sa_mask);
-  action.sa_flags = SA_RESTART | SA_NOCLDSTOP;
-  sigaction (SIGCHLD, &action, NULL);
+  signal (SIGCHLD, g_child_watch_signal_handler);
 }
 
 static gpointer
@@ -3622,7 +3618,6 @@ child_watch_helper_thread (gpointer data)
 static void
 g_child_watch_source_init_multi_threaded (void)
 {
-  struct sigaction action;
   GError *error = NULL;
 
   g_assert (g_thread_supported());
@@ -3636,11 +3631,7 @@ g_child_watch_source_init_multi_threaded (void)
   if (g_thread_create (child_watch_helper_thread, NULL, FALSE, &error) == NULL)
     g_error ("Cannot create a thread to monitor child exit status: %s\n", error->message);
   child_watch_init_state = CHILD_WATCH_INITIALIZED_THREADED;
-
-  action.sa_handler = g_child_watch_signal_handler;
-  sigemptyset (&action.sa_mask);
-  action.sa_flags = SA_RESTART | SA_NOCLDSTOP;
-  sigaction (SIGCHLD, &action, NULL);
+  signal (SIGCHLD, g_child_watch_signal_handler);
 }
 
 static void
@@ -3725,8 +3716,6 @@ g_child_watch_source_new (GPid pid)
  * source is still active. Typically, you will want to call
  * g_spawn_close_pid() in the callback function for the source.
  * 
- * GLib supports only a single callback per process id.
- *
  * Return value: the id of event source.
  *
  * Since: 2.4
@@ -3769,8 +3758,6 @@ g_child_watch_add_full (gint            priority,
  * source is still active. Typically, you will want to call
  * g_spawn_close_pid() in the callback function for the source.
  *
- * GLib supports only a single callback per process id.
- *
  * Return value: the id of event source.
  *
  * Since: 2.4
