@@ -21,6 +21,7 @@
 
 #include "goption.h"
 
+#include "galias.h"
 #include "glib.h"
 #include "gi18n.h"
 
@@ -133,7 +134,8 @@ g_option_context_free (GOptionContext *context)
   g_list_foreach (context->groups, (GFunc)g_option_group_free, NULL);
   g_list_free (context->groups);
 
-  g_option_group_free (context->main_group);
+  if (context->main_group) 
+    g_option_group_free (context->main_group);
 
   free_changes_list (context, FALSE);
   free_pending_nulls (context, FALSE);
@@ -256,9 +258,9 @@ print_help (GOptionContext *context,
   GList *list;
   gint max_length, len;
   gint i;
-
+  
   g_print ("%s\n  %s %s %s\n\n", 
-	   _("Usage:"), g_get_prgname (), _("[OPTION...]"),
+	   _("Usage:"), g_get_prgname(), _("[OPTION...]"),
 	   context->parameter_string ? context->parameter_string : "");
 
   list = context->groups;
@@ -357,8 +359,9 @@ print_help (GOptionContext *context,
 
       g_print ("%s\n", _("Application Options:"));
 
-      for (i = 0; i < context->main_group->n_entries; i++) 
-	print_entry (context->main_group, max_length, &context->main_group->entries[i]);
+      if (context->main_group)
+	for (i = 0; i < context->main_group->n_entries; i++) 
+	  print_entry (context->main_group, max_length, &context->main_group->entries[i]);
 
       while (list != NULL)
 	{
@@ -820,6 +823,24 @@ g_option_context_parse (GOptionContext   *context,
   gint i, j, k;
   GList *list;
 
+  /* Set program name */
+  if (argc && argv)
+    {
+      gchar *prgname;
+      
+      prgname = strrchr ((*argv)[0], G_DIR_SEPARATOR);
+      if (prgname)
+	prgname++;
+      else
+	prgname = (*argv)[0];
+
+      g_set_prgname (prgname);
+    }
+  else
+    {
+      g_set_prgname ("<unknown>");
+    }
+  
   /* Call pre-parse hooks */
   list = context->groups;
   while (list)
@@ -836,7 +857,7 @@ g_option_context_parse (GOptionContext   *context,
       list = list->next;
     }
 
-  if (context->main_group->pre_parse_func)
+  if (context->main_group && context->main_group->pre_parse_func)
     {
       if (!(* context->main_group->pre_parse_func) (context, context->main_group,
 						    context->main_group->user_data, error))
@@ -890,7 +911,8 @@ g_option_context_parse (GOptionContext   *context,
 			}
 		    }
 
-		  if (!parse_long_option (context, context->main_group, &i, arg,
+		  if (context->main_group &&
+		      !parse_long_option (context, context->main_group, &i, arg,
 					  argc, argv, error, &parsed))
 		    goto fail;
 
@@ -932,7 +954,8 @@ g_option_context_parse (GOptionContext   *context,
 		    {
 		      parsed = FALSE;
 		      
-		      if (!parse_short_option (context, context->main_group,
+		      if (context->main_group &&
+			  !parse_short_option (context, context->main_group,
 					       i, &new_i, arg[j],
 					       argc, argv, error, &parsed))
 			{
@@ -1024,7 +1047,7 @@ g_option_context_parse (GOptionContext   *context,
 	  list = list->next;
 	}
 
-      if (context->main_group->post_parse_func)
+      if (context->main_group && context->main_group->post_parse_func)
 	{
 	  if (!(* context->main_group->post_parse_func) (context, context->main_group,
 							 context->main_group->user_data, error))
@@ -1046,8 +1069,7 @@ g_option_context_parse (GOptionContext   *context,
 		(*argv)[j-k] = (*argv)[j];
 	      *argc -= k;
 	    }
-	}
-      
+	}      
     }
 
   return TRUE;
@@ -1067,7 +1089,7 @@ g_option_context_parse (GOptionContext   *context,
       list = list->next;
     }
 
-  if (context->main_group->error_func)
+  if (context->main_group && context->main_group->error_func)
     (* context->main_group->error_func) (context, context->main_group,
 					 context->main_group->user_data, error);
   
